@@ -1,7 +1,7 @@
 ## SimInf, a framework for stochastic disease spread simulations
 ## Copyright (C) 2015  Pavol Bauer
-## Copyright (C) 2015 - 2017  Stefan Engblom
-## Copyright (C) 2015 - 2017  Stefan Widgren
+## Copyright (C) 2015 - 2018  Stefan Engblom
+## Copyright (C) 2015 - 2018  Stefan Widgren
 ##
 ## This program is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -22,15 +22,15 @@
 ##' @param x Value to check
 ##' @param tol Tolerance of the check
 ##' @return logical vector
-##' @keywords internal
-is_wholenumber <- function(x, tol = .Machine$double.eps^0.5)
-{
+##' @noRd
+is_wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
     abs(x - round(x)) < tol
 }
 
 ##' Class \code{"SimInf_events"}
 ##'
-##' Class to handle the scheduled events
+##' Class to hold data for scheduled events to modify the discrete
+##' state of individuals in a node at a pre-defined time t.
 ##' @section Slots:
 ##' \describe{
 ##'   \item{E}{
@@ -40,22 +40,31 @@ is_wholenumber <- function(x, tol = .Machine$double.eps^0.5)
 ##'     transfer} and \emph{external transfer} events, a non-zero
 ##'     entry indicate the compartments to sample individuals from.
 ##'     For the \emph{enter} event, all individuals enter first
-##'     non-zero compartment. Sparse matrix of object class
-##'     \code{"\linkS4class{dgCMatrix}"}.
+##'     non-zero compartment. \code{E} is sparse matrix of class
+##'     \code{\linkS4class{dgCMatrix}}.
 ##'   }
 ##'   \item{N}{
-##'     Each row represents one compartment in the model and the
-##'     values determine how to move sampled individuals in
-##'     \emph{internal transfer} and \emph{external transfer} events.
+##'      Determines how individuals in \emph{internal transfer} and
+##'      \emph{external transfer} events are shifted to enter another
+##'      compartment.  Each row corresponds to one compartment in the
+##'      model.  The values in a column are added to the current
+##'      compartment of sampled individuals to specify the destination
+##'      compartment, for example, a value of \code{1} in an entry
+##'      means that sampled individuals in this compartment are moved
+##'      to the next compartment.  Which column to use for each event
+##'      is specified by the \code{shift} vector (see below).
+##'      \code{N} is an integer matrix.
 ##'   }
 ##'   \item{event}{
 ##'     Type of event: 0) \emph{exit}, 1) \emph{enter}, 2)
 ##'     \emph{internal transfer}, and 3) \emph{external transfer}.
 ##'     Other values are reserved for future event types and not
-##'     supported by the current default core solver. Integer vector.
+##'     supported by the current solvers. Integer vector.
 ##'   }
 ##'   \item{time}{
-##'     Time for the event. Integer vector.
+##'     Time of when the event occurs i.e., the event is processed
+##'     when time is reached in the simulation.  \code{time} is an
+##'     integer vector.
 ##'   }
 ##'   \item{node}{
 ##'     The node that the event operates on. Also the source node for
@@ -63,9 +72,11 @@ is_wholenumber <- function(x, tol = .Machine$double.eps^0.5)
 ##'     1 <= \code{node[i]} <= Number of nodes.
 ##'   }
 ##'   \item{dest}{
-##'     The destination node for an \emph{external transfer} event.
-##'     Should be \code{0} for the other event types.
-##'     Integer vector. dest[i] >= 0.
+##'     The destination node for an \emph{external transfer} event
+##'     i.e., individuals are moved from \code{node} to \code{dest},
+##'     where 1 <= \code{dest[i]} <= Number of nodes.  Set \code{event
+##'     = 0} for the other event types.  \code{dest} is an integer
+##'     vector.
 ##'   }
 ##'   \item{n}{
 ##'     The number of individuals affected by the event. Integer vector.
@@ -79,19 +90,25 @@ is_wholenumber <- function(x, tol = .Machine$double.eps^0.5)
 ##'     0 <= proportion[i] <= 1.
 ##'   }
 ##'   \item{select}{
-##'     The column \code{j} in the event matrix \code{E} that
-##'     determines the compartments that the event operates
-##'     on. Integer vector.
+##'     To process \code{event[i]}, the compartments affected by the
+##'     event are specified with \code{select[i]} together with the
+##'     matrix \code{E}, where \code{select[i]} determines which
+##'     column in \code{E} to use.  The specific individuals affected
+##'     by the event are proportionally sampled from the compartments
+##'     corresponding to the non-zero entries in the specified column
+##'     in \code{E[, select[i]]}, where \code{select} is an integer
+##'     vector.
 ##'   }
 ##'   \item{shift}{
-##'     The column \code{k} in the shift matrix \code{N} that
-##'     determines how individuals in \emph{internal transfer} and
+##'     Determines how individuals in \emph{internal transfer} and
 ##'     \emph{external transfer} events are shifted to enter another
-##'     compartment.  Should be \code{0} for the other event types.
-##'     Integer vector.
+##'     compartment.  The sampled individuals are shifted according to
+##'     column \code{shift[i]} in matrix \code{N} i.e., \code{N[,
+##'     shift[i]]}, where \code{shift} is an integer vector.  See
+##'     above for a description of \code{N}. Unsued for the other
+##'     event types.
 ##'   }
 ##' }
-##' @keywords methods
 ##' @export
 setClass("SimInf_events",
          slots = c(E          = "dgCMatrix",
@@ -105,7 +122,11 @@ setClass("SimInf_events",
                    select     = "integer",
                    shift      = "integer"),
          validity = function(object) {
-             errors <- character()
+             ## Check that E and N have identical compartments
+             if ((dim(object@E)[1] > 0) && (dim(object@N)[1] > 0)) {
+                 if (!identical(rownames(object@E), rownames(object@N)))
+                     return("'E' and 'N' must have identical compartments")
+             }
 
              if (!identical(length(unique(c(length(object@event),
                                             length(object@time),
@@ -115,63 +136,54 @@ setClass("SimInf_events",
                                             length(object@proportion),
                                             length(object@select),
                                             length(object@shift)))) , 1L)) {
-                 errors <- c(errors,
-                             "All scheduled events must have equal length.")
+                 return("All scheduled events must have equal length.")
              }
 
-             if (!all(object@time > 0)) {
-                 errors <- c(errors,
-                             "time must be greater than 0")
-             }
+             if (!all(object@time > 0))
+                 return("time must be greater than 0")
 
-             if (any(object@event < 0, object@event > 3)) {
-                 errors <- c(errors,
-                             "event must be in the range 0 <= event <= 3")
-             }
+             if (any(object@event < 0, object@event > 3))
+                 return("event must be in the range 0 <= event <= 3")
 
-             if (any(object@node < 1)) {
-                 errors <- c(errors,
-                             "'node' must be greater or equal to 1")
-             }
+             if (any(object@node < 1))
+                 return("'node' must be greater or equal to 1")
 
-             if (any(object@dest[object@event == 3] < 1)) {
-                 errors <- c(errors,
-                             "'dest' must be greater or equal to 1")
-             }
+             if (any(object@dest[object@event == 3] < 1))
+                 return("'dest' must be greater or equal to 1")
 
-             if (any(object@proportion < 0, object@proportion > 1)) {
-                 errors <- c(errors,
-                             "prop must be in the range 0 <= prop <= 1")
-             }
+             if (any(object@proportion < 0, object@proportion > 1))
+                 return("prop must be in the range 0 <= prop <= 1")
 
-             if (any(object@select < 1, object@select > dim(object@E)[2])) {
-                 errors <- c(errors,
-                             "select must be in the range 1 <= select <= Nselect")
-             }
+             if (any(object@select < 1, object@select > dim(object@E)[2]))
+                 return("select must be in the range 1 <= select <= Nselect")
 
-             if (any(object@shift[object@event == 2] < 1)) {
-                 errors <- c(errors,
-                             "'shift' must be greater or equal to 1")
-             }
+             if (any(object@shift[object@event == 2] < 1))
+                 return("'shift' must be greater or equal to 1")
 
-             if (length(errors) == 0) TRUE else errors
+             TRUE
          }
 )
 
-##' Create a \code{"\linkS4class{SimInf_events}"} object
+##' Create a \code{\linkS4class{SimInf_events}} object
 ##'
 ##' The argument events must be a \code{data.frame} with the following
 ##' columns:
 ##' \describe{
 ##'   \item{event}{
-##'     Type of event: 0) \emph{exit}, 1) \emph{enter}, 2)
-##'     \emph{internal transfer}, and 3) \emph{external transfer}.
-##'     Other values are reserved for future event types and not
-##'     supported by the current default core solver.
+##'     Four event types are supported by the current solvers:
+##'     \emph{exit}, \emph{enter}, \emph{internal transfer}, and
+##'     \emph{external transfer}.  When assigning the events, they can
+##'     either be coded as a numerical value or a character string:
+##'     \emph{exit;} \code{0} or \code{'exit'}, \emph{enter;} \code{1}
+##'     or \code{'enter'}, \emph{internal transfer;} \code{2} or
+##'     \code{'intTrans'}, and \emph{external transfer;} \code{3} or
+##'     \code{'extTrans'}.  Internally in \pkg{SimInf}, the event type
+##'     is coded as a numerical value.
 ##'   }
 ##'   \item{time}{
-##'     Time for the event. Can be either an \code{integer} or a
-##'     \code{Date} vector.  A \code{Date} vector is coerced to a
+##'     When the event occurs i.e., the event is processed when time
+##'     is reached in the simulation. Can be either an \code{integer}
+##'     or a \code{Date} vector.  A \code{Date} vector is coerced to a
 ##'     numeric vector as days, where \code{t0} determines the offset
 ##'     to match the time of the events to the model \code{tspan}
 ##'     vector.
@@ -182,8 +194,11 @@ setClass("SimInf_events",
 ##'     1 <= \code{node[i]} <= Number of nodes.
 ##'   }
 ##'   \item{dest}{
-##'     The destination node for an \emph{external transfer} event.
-##'     Should be \code{0} for the other event types. dest[i] >= 0.
+##'     The destination node for an \emph{external transfer} event
+##'     i.e., individuals are moved from \code{node} to \code{dest},
+##'     where 1 <= \code{dest[i]} <= Number of nodes.  Set \code{event
+##'     = 0} for the other event types.  \code{dest} is an integer
+##'     vector.
 ##'   }
 ##'   \item{n}{
 ##'     The number of individuals affected by the event. n[i] >= 0.
@@ -195,23 +210,44 @@ setClass("SimInf_events",
 ##'     with \code{proportion[i]}. 0 <= proportion[i] <= 1.
 ##'   }
 ##'   \item{select}{
-##'     The column \code{j} in the event matrix \code{E} that
-##'     determines the compartments that the event operates on.
+##'     To process \code{event[i]}, the compartments affected by the
+##'     event are specified with \code{select[i]} together with the
+##'     matrix \code{E}, where \code{select[i]} determines which
+##'     column in \code{E} to use.  The specific individuals affected
+##'     by the event are proportionally sampled from the compartments
+##'     corresponding to the non-zero entries in the specified column
+##'     in \code{E[, select[i]]}, where \code{select} is an integer
+##'     vector.
 ##'   }
 ##'   \item{shift}{
-##'     The column \code{k} in the shift matrix \code{N} that
-##'     determines how individuals in \emph{internal transfer} and
+##'     Determines how individuals in \emph{internal transfer} and
 ##'     \emph{external transfer} events are shifted to enter another
-##'     compartment.  Should be \code{0} for the other event types.
+##'     compartment.  The sampled individuals are shifted according to
+##'     column \code{shift[i]} in matrix \code{N} i.e., \code{N[,
+##'     shift[i]]}, where \code{shift} is an integer vector.  See
+##'     above for a description of \code{N}. Unsued for the other
+##'     event types.
 ##'   }
 ##' }
 ##'
-##' @param E Sparse matrix of object class
-##'     \code{"\linkS4class{dgCMatrix}"} that selects the states to
-##'     include for sampling in an event.
-##' @param N Sparse matrix of object class
-##'     \code{"\linkS4class{dgCMatrix}"} that determines how to shift
-##'     the states in an \code{INTERNAL_TRANSFER_EVENT}.
+##' @param E Each row corresponds to one compartment in the model. The
+##'     non-zero entries in a column indicates the compartments to
+##'     include in an event.  For the \emph{exit}, \emph{internal
+##'     transfer} and \emph{external transfer} events, a non-zero
+##'     entry indicate the compartments to sample individuals from.
+##'     For the \emph{enter} event, all individuals enter first
+##'     non-zero compartment. \code{E} is sparse matrix of class
+##'     \code{\linkS4class{dgCMatrix}}.
+##' @param N Determines how individuals in \emph{internal transfer}
+##'     and \emph{external transfer} events are shifted to enter
+##'     another compartment.  Each row corresponds to one compartment
+##'     in the model.  The values in a column are added to the current
+##'     compartment of sampled individuals to specify the destination
+##'     compartment, for example, a value of \code{1} in an entry
+##'     means that sampled individuals in this compartment are moved
+##'     to the next compartment.  Which column to use for each event
+##'     is specified by the \code{shift} vector (see below).  \code{N}
+##'     is an integer matrix.
 ##' @param events A \code{data.frame} with events.
 ##' @param t0 If \code{events$time} is a \code{Date} vector, then
 ##'     \code{t0} determines the offset to match the time of the
@@ -220,24 +256,26 @@ setClass("SimInf_events",
 ##'     \code{NULL}.
 ##' @return S4 class \code{SimInf_events}
 ##' @export
+##' @importFrom methods as
+##' @importFrom methods is
+##' @importFrom methods new
 SimInf_events <- function(E      = NULL,
-                             N      = NULL,
-                             events = NULL,
-                             t0     = NULL)
+                          N      = NULL,
+                          events = NULL,
+                          t0     = NULL)
 {
     ## Check E
     if (is.null(E)) {
         if (!is.null(events))
             stop("events is not NULL when E is NULL")
-        E <- methods::new("dgCMatrix")
+        E <- new("dgCMatrix")
+    } else if (!is(E, "dgCMatrix")) {
+        E <- as(E, "dgCMatrix")
     }
 
     ## Check N
-    if (is.null(N)) {
-        if (!is.null(events))
-            stop("events is not NULL when N is NULL")
+    if (is.null(N))
         N <- matrix(integer(0), nrow = 0, ncol = 0)
-    }
     if (!all(is.matrix(N), is.numeric(N)))
         stop("'N' must be an integer matrix")
     if (!is.integer(N)) {
@@ -259,17 +297,33 @@ SimInf_events <- function(E      = NULL,
     }
     if (!is.data.frame(events))
         stop("events must be a data.frame")
-    if (!identical(ncol(events), 8L))
-        stop("Wrong dimension of events")
     if (!all(c("event", "time", "node", "dest",
                "n", "proportion", "select",
                "shift") %in% names(events))) {
         stop("Missing columns in events")
     }
 
+    ## Do we have to recode the event type as a numerical value
+    if (is.character(events$event) || is.factor(events$event)) {
+        if (!all(events$event %in% c("enter", "exit", "extTrans", "intTrans")))
+            stop("'event' type must be 'enter', 'exit', 'extTrans' or 'intTrans'")
+
+        ## Find indices to 'enter', 'internal transfer' and 'external
+        ## transfer' events.
+        i_enter <- which(events$event == "enter")
+        i_intTrans <- which(events$event == "intTrans")
+        i_extTrans <- which(events$event == "extTrans")
+
+        ## Replace the character event type with a numerical value.
+        events$event <- rep(0L, nrow(events))
+        events$event[i_enter] <- 1L
+        events$event[i_intTrans] <- 2L
+        events$event[i_extTrans] <- 3L
+    }
+
     ## Check time
     if (nrow(events)) {
-        if (methods::is(events$time, "Date")) {
+        if (is(events$time, "Date")) {
             if (is.null(t0))
                 stop("Missing 't0'")
             if (!all(identical(length(t0), 1L), is.numeric(t0)))
@@ -306,19 +360,34 @@ SimInf_events <- function(E      = NULL,
 
     events <- events[order(events$time, events$event, events$select),]
 
-    return(methods::new("SimInf_events",
-                        E          = E,
-                        N          = N,
-                        event      = as.integer(events$event),
-                        time       = as.integer(events$time),
-                        node       = as.integer(events$node),
-                        dest       = as.integer(events$dest),
-                        n          = as.integer(events$n),
-                        proportion = as.numeric(events$proportion),
-                        select     = as.integer(events$select),
-                        shift      = as.integer(events$shift)))
+    new("SimInf_events",
+        E          = E,
+        N          = N,
+        event      = as.integer(events$event),
+        time       = as.integer(events$time),
+        node       = as.integer(events$node),
+        dest       = as.integer(events$dest),
+        n          = as.integer(events$n),
+        proportion = as.numeric(events$proportion),
+        select     = as.integer(events$select),
+        shift      = as.integer(events$shift))
 }
 
+setAs(from = "SimInf_events",
+      to = "data.frame",
+      def = function(from)
+      {
+          data.frame(event = from@event,
+                     time = from@time,
+                     node = from@node,
+                     dest = from@dest,
+                     n = from@n,
+                     proportion = from@proportion,
+                     select = from@select,
+                     shift = from@shift)
+      }
+)
+
 ##' Plot scheduled events
 ##'
 ##' @param x the time points of the events.
@@ -327,7 +396,9 @@ SimInf_events <- function(E      = NULL,
 ##' @param frame.plot a logical indicating whether a box should be
 ##'     drawn around the plot.
 ##' @param ... additional arguments affecting the plot.
-##' @keywords internal
+##' @importFrom graphics plot
+##' @importFrom graphics mtext
+##' @noRd
 plot_SimInf_events <- function(x,
                                   y,
                                   events = c("Exit",
@@ -353,35 +424,38 @@ plot_SimInf_events <- function(x,
             y <- rep(0, length(x))
         }
 
-        graphics::plot(x, y, type = "l", ylim = ylim, xlab = "",
-                       ylab = "", frame.plot = frame.plot, ...)
+        plot(x, y, type = "l", ylim = ylim, xlab = "",
+             ylab = "", frame.plot = frame.plot, ...)
     } else {
-        graphics::plot(0, 0, type = "n", xlab = "", ylab = "",
-                       frame.plot = frame.plot, ...)
+        plot(0, 0, type = "n", xlab = "", ylab = "",
+             frame.plot = frame.plot, ...)
     }
 
-    graphics::mtext(events, side = 3, line = 0)
-    graphics::mtext("Individuals", side = 2, line = 2)
-    graphics::mtext("Time", side = 1, line = 2)
+    mtext(events, side = 3, line = 0)
+    mtext("Individuals", side = 2, line = 2)
+    mtext("Time", side = 1, line = 2)
 }
 
-##' Plot \code{\linkS4class{SimInf_events}}
+##' Display the distribution of scheduled events over time
 ##'
+##' @param x The events data to plot.
 ##' @param frame.plot Draw a frame around each plot. Default is FALSE.
-##' @name plot-methods
-##' @aliases plot plot-methods plot,SimInf_events-method
+##' @param ... Additional arguments affecting the plot
+##' @aliases plot,SimInf_events-method
 ##' @export
+##' @importFrom graphics par
+##' @importFrom stats xtabs
 setMethod("plot",
           signature(x = "SimInf_events"),
           function(x, frame.plot = FALSE, ...)
           {
-              savepar <- graphics::par(mfrow = c(2, 2),
-                                       oma = c(1, 1, 2, 0),
-                                       mar = c(4, 3, 1, 1))
-              on.exit(graphics::par(savepar))
+              savepar <- par(mfrow = c(2, 2),
+                             oma = c(1, 1, 2, 0),
+                             mar = c(4, 3, 1, 1))
+              on.exit(par(savepar))
 
-              yy <- stats::xtabs(n ~ event + time,
-                         cbind(event = x@event, time = x@time, n = x@n))
+              yy <- xtabs(n ~ event + time,
+                          cbind(event = x@event, time = x@time, n = x@n))
               xx <- as.integer(colnames(yy))
 
               ## Plot events
@@ -395,10 +469,8 @@ setMethod("plot",
 ##' Brief summary of \code{SimInf_events}
 ##'
 ##' Shows the number of scheduled events.
-##' @aliases show,SimInf_events-methods
 ##' @param object The SimInf_events \code{object}
 ##' @return None (invisible 'NULL').
-##' @keywords methods
 ##' @export
 ##' @importFrom methods show
 setMethod("show",
@@ -410,15 +482,13 @@ setMethod("show",
           }
 )
 
-##' Summary of \code{SimInf_events}
+##' Detailed summary of a \code{SimInf_events} object
 ##'
 ##' Shows the number of scheduled events and the number of scheduled
 ##' events per event type.
-##' @aliases summary,SimInf_events-methods
 ##' @param object The \code{SimInf_events} object
 ##' @param ... Additional arguments affecting the summary produced.
 ##' @return None (invisible 'NULL').
-##' @keywords methods
 ##' @export
 setMethod("summary",
           signature(object = "SimInf_events"),
