@@ -127,12 +127,12 @@ int SISe_post_time_step(
  *
  * @param model The SISe model.
  * @param threads Number of threads.
- * @param seed Random number seed.
+ * @param solver The numerical solver.
  * @return The simulated trajectory.
  */
-SEXP SISe_run(SEXP model, SEXP threads, SEXP seed)
+SEXP SISe_run(SEXP model, SEXP threads, SEXP solver)
 {
     TRFun tr_fun[] = {&SISe_S_to_I, &SISe_I_to_S};
 
-    return SimInf_run(model, threads, seed, tr_fun, &SISe_post_time_step);
+    return SimInf_run(model, threads, solver, tr_fun, &SISe_post_time_step);
 }
